#!/usr/bin/python
# -*- coding:utf-8 -*-

"""An abstract class of scoreboard classes.
"""

from dom import *
import scorecard 

def newInstance():
    return MotoGP_yahoo_UK()


class CategoryLoader:
    def __init__( self, category, url ):
        self.category = category
        self.url = url
        self.table = None
        pass

    def __repr__( self ):
        return "\"%s\"\t\"%s\"" % ( self.category, self.url )

    def is_race( self ):
        import re
        if re.search( "\\s+Race\\s*$", self.category ):
            return True
        return False

    def load( self ):
        #print "laoding " + self.url
        parser = DOMConverter()
        self.table = []
        import loader
        data = loader.loadHTML( self.url )
        parser.feed( data )
        document = parser.get_document()
        elem_table = document.get_element_at_path( "/html/body/table/tr/td/table" )
        elem_row = elem_table.get_element_at_path( "tbody/tr,2" )
        #elem_row = elem_row.next_sibling( 'tr' )
        
        while elem_row:
            items = elem_row.get_text( "-*-" ).split( "-*-" )
            if len( items ) >= 6:
                #p, num, driver, team, time, deficit = items[ 0: 6 ]
                self.table.append( items[ 0 : 6 ] )
                pass
            try:
                elem_row = elem_row.next_sibling( 'tr' )
            except:
                break
            pass
        pass

    def toHTML( self ):
        if self.table == None:
            return ""
        content = "<span class=\"category\">%s</span>\n" % self.category
        content += '<table>\n'
        content += '<tr><td class="position">P.</td><td class="number">No.</td><td class="driver">Driver</td><td class="team">Team</td><td class="time">Time/Lap</td><td class="deficit">Deficit</td></tr>\n'
        for row in self.table:
            if row[ 5 ].find( "NaN" ) >= 0:
                row[ 5 ] = "-"
                pass
            content += '<tr><td class="position">%s</td>' % row[ 0 ]\
                       + '<td class="number">%s</td>' % row[ 1 ] \
                       + '<td class="driver">%s</td>' % row[ 2 ]\
                       + '<td class="team">%s</td>' % row[ 3 ]\
                       + '<td class="time">%s</td>' % row[ 4 ]\
                       + '<td class="deficit">%s</td>' % row[ 5 ]\
                       + '</tr>\n'
            pass
        content += '</table>\n'
        return content

    pass

import sys

#sys.exit()

class MotoGP_yahoo_UK ( scorecard.ScoreCard ):
    def __init__( self ):
        scorecard.ScoreCard.__init__( self )
        return

    def getSportsCategory( self ):
        return "autoracing"

    def getCountry( self ):
        return "World"

    def getName( self ):
        return "MotoGP (uk.yahoo.com)"

    def getVersion( self ):
        """Version of the mudule."""
        return "1.0"

    def getIcon( self, team ):
        """URI (local resource is recommended, see makelogo.py) of
        a team logo.
        This method returns empty string (not None) if no URI are
        found."""
        return ""

    def getEncoding( self ):
        return "latin1"

    def applyFilter( self, contents ):
        """To eliminate malformed HTML, you can set filters
        in this method"""
        return contents

    def decode( self, contents ):
        """Decoding method of given texts to UTF-8."""
        return contents
        
    def getLanguage( self ):
        return "en"

    def getURL( self ):
        #import tempfile
        return "http://uk.sports.yahoo.com/mo/"

    def interpreteDocument( self, document ):
        import re, sys
        #print document
        categories = []
        elem = document.get_element_at_path( "/html/body/table/tr/td/table/tr/td/table/tbody/tr/th/table/tbody/tr/th/table/tbody/tr/th/div/li" )
        while elem:
            
            src = "http://uk.sports.yahoo.com" + elem.get_element_at_path( 'a' ).get_attribute( 'href' )
            content = elem.get_text()
            if src[ 0 ] == '/':
                src = "http://uk.sports.yahoo.com" + src
            elif src[ 0: 7 ] == 'http://':
                src = src
            else:
                src = self.getURL() + src
                pass
            categories.append( CategoryLoader( content, src ) )
            #print elem.get_element_at_path( 'a' ).get_attribute( 'href' )
            try:
                elem = elem.next_sibling( 'li' )
            except:
                break
            pass
        for category in categories:
            if category.is_race():
                category.load()
                pass
            pass
        self.categories = categories
        pass

    def getIgnorableTags( self ):
        return [ 'small', 'ul', 'span', 'p' ]

    def toHTML( self ):
        """Format data into HTML.
        This method must be overridden."""
        html  = self.html_header()
        for category in self.categories:
            html += category.toHTML()
            pass
        html += self.html_footer()
        return html

    def html_css_plugin( self ):
        return """<style type=\"text/css\">
        .position { width:15px; }
        .number { width:15px;}
        .driver { width:80px; align:center; font-weight:bold;}
        .team {width:60px;}
        .time {width:60px;}
        .deficit{width:70px;}
        </style>
        """

    pass


if __name__ == "__main__":
    
    import sys
    import tempfile
    from dom import *

    skin = None
    filename_input = None

    obj = newInstance()
    if len( sys.argv ) > 1: filename_input = sys.argv[ 1 ]
    
    if obj == None or obj.isAvailable() == False:
        print "invalid object"
        sys.exit()

    url = obj.getURL()
    encoding = obj.getEncoding()
    if filename_input:
        data = ""
        for line in open( filename_input ).readlines():
            data += line
            pass
        pass
    else:
        import loader
        data = loader.loadHTML( url, encoding )
        pass
    kw_start = obj.getStartKeyword()
    kw_end   = obj.getEndKeyword()
    parser = DOMConverter()
    for tag in obj.getIgnorableTags():
        parser.add_ignoring_tag( tag )
        pass
    if kw_start == None or kw_end == None:
        if encoding == None:
            parser.feed( obj.decode( obj.applyFilter( data ) ) )
        else:
            for line in data.split( "\n" ):
                parser.feed( obj.applyFilter( line ) )
                pass
            pass
        pass
    else:
        flag_inside = False
        parser.feed( '<html>' )
        for line in re.split( "\n", data ):
            line = obj.applyFilter( line )
            if encoding == None:
                line = obj.decode( line.strip() )
                pass
            if flag_inside:
                if line.count( kw_end ) > 0:
                    flag_inside = False
                    break
                else:
                    parser.feed( line )
                    pass
            elif line.count( kw_start ) > 0: flag_inside = True
            pass
        parser.feed( '</html>' )
        pass
    document = parser.get_document()
    obj.interpreteDocument( document )
    contents = obj.toHTML()
    print contents
